@@ -1,11 +1,7 @@
 package Selenium::Remote::RemoteConnection;
-{
-  $Selenium::Remote::RemoteConnection::VERSION = '0.17';
-}
-
-use strict;
-use warnings;
-
+$Selenium::Remote::RemoteConnection::VERSION = '0.18';
+use Moo;
+use Try::Tiny;
 use LWP::UserAgent;
 use HTTP::Headers;
 use HTTP::Request;
@@ -13,32 +9,46 @@ use Net::Ping;
 use Carp qw(croak);
 use JSON;
 use Data::Dumper;
-
 use Selenium::Remote::ErrorHandler;
 
-sub new {
-    my ($class, $remote_srvr, $port) = @_;
-    
-    my $self = {
-                 remote_server_addr => $remote_srvr,
-                 port               => $port,
-                 debug              => 0,
+has 'remote_server_addr' => (
+    is => 'rw',
+);
+
+has 'port' => (
+    is => 'rw',
+);
+
+has 'debug' => (
+    is => 'rw',
+    default => sub { 0 }
+);
+
+has 'ua' => (
+    is => 'lazy',
+    builder => sub { return LWP::UserAgent->new; }
+);
+
+sub BUILD {
+    my $self = shift;
+    my $status;
+    try {
+      $status = $self->request('GET','status');
+    }
+    catch {
+        croak "Could not connect to SeleniumWebDriver: $_" ;
     };
-    bless $self, $class or die "Can't bless $class: $!";
-    my $status = eval {$self->request('GET','status');};
-    croak "Could not connect to SeleniumWebDriver" if($@);
     if($status->{cmd_status} ne 'OK') {
         # Could be grid, see if we can talk to it
         $status = undef;
         $status = $self->request('GET', 'grid/api/testsession');
     }
-    if($status->{cmd_status} eq 'OK') {
-        return $self;
-    } else {
+    unless ($status->{cmd_status} eq 'OK') {
         croak "Selenium server did not return proper status";
     }
 }
 
+
 # This request method is tailored for Selenium RC server
 sub request {
     my ($self, $method, $url, $params) = @_;
@@ -52,33 +62,32 @@ sub request {
     elsif ($url =~ m/grid/g) {
         $fullurl =
             "http://"
-          . $self->{remote_server_addr} . ":"
-          . $self->{port}
+          . $self->remote_server_addr . ":"
+          . $self->port
           . "/$url";
     }
     else {
         $fullurl =
             "http://"
-          . $self->{remote_server_addr} . ":"
-          . $self->{port}
+          . $self->remote_server_addr . ":"
+          . $self->port
           . "/wd/hub/$url";
     }
 
     if ((defined $params) && $params ne '') {
-        my $json = new JSON;
+        my $json = JSON->new;
         $json->allow_blessed;
         $content = $json->allow_nonref->utf8->encode($params);
     }
-    
-    print "REQ: $url, $content\n" if $self->{debug};
+
+    print "REQ: $url, $content\n" if $self->debug;
 
     # HTTP request
-    my $ua = LWP::UserAgent->new;
     my $header =
       HTTP::Headers->new(Content_Type => 'application/json; charset=utf-8');
     $header->header('Accept' => 'application/json');
     my $request = HTTP::Request->new($method, $fullurl, $header, $content);
-    my $response = $ua->request($request);
+    my $response = $self->ua->request($request);
 
     return $self->_process_response($response);
 }
@@ -86,14 +95,14 @@ sub request {
 sub _process_response {
     my ($self, $response) = @_;
     my $data; # server response 'value' that'll be returned to the user
-    my $json = new JSON;
+    my $json = JSON->new;
 
     if ($response->is_redirect) {
         return $self->request('GET', $response->header('location'));
     }
     else {
         my $decoded_json = undef;
-        print "RES: ".$response->decoded_content."\n\n" if $self->{debug};
+        print "RES: ".$response->decoded_content."\n\n" if $self->debug;
         if (($response->message ne 'No Content') && ($response->content ne '')) {
             if ($response->content_type !~ m/json/i) {
                 $data->{'cmd_return'} = 'Server returned error message '.$response->content.' instead of data';
@@ -102,15 +111,15 @@ sub _process_response {
             $decoded_json = $json->allow_nonref(1)->utf8(1)->decode($response->content);
             $data->{'sessionId'} = $decoded_json->{'sessionId'};
         }
-        
+
         if ($response->is_error) {
-            my $error_handler = new Selenium::Remote::ErrorHandler;
+            my $error_handler = Selenium::Remote::ErrorHandler->new;
             $data->{'cmd_status'} = 'NOTOK';
             if (defined $decoded_json) {
                 $data->{'cmd_return'} = $error_handler->process_error($decoded_json);
             }
             else {
-                $data->{'cmd_return'} = 'Server returned error code '.$response->code.' and no data';          
+                $data->{'cmd_return'} = 'Server returned error code '.$response->code.' and no data';
             }
             return $data;
         }
@@ -120,7 +129,7 @@ sub _process_response {
                 $data->{'cmd_return'} = $decoded_json->{'value'};
             }
             else {
-                $data->{'cmd_return'} = 'Server returned status code '.$response->code.' but no data';          
+                $data->{'cmd_return'} = 'Server returned status code '.$response->code.' but no data';
             }
             return $data;
         }
@@ -146,7 +155,7 @@ Selenium::Remote::RemoteConnection - Connect to a selenium server
 
 =head1 VERSION
 
-version 0.17
+version 0.18
 
 =head1 SEE ALSO
 
@@ -156,11 +165,11 @@ L<http://code.google.com/p/selenium/>.
 =head1 BUGS
 
 The Selenium issue tracking system is available online at
-L<http://github.com/aivaturi/Selenium-Remote-Driver/issues>.
+L<http://github.com/gempesaw/Selenium-Remote-Driver/issues>.
 
 =head1 CURRENT MAINTAINER
 
-Charles Howes C<< <chowes@cpan.org> >>
+Daniel Gempesaw C<< <gempesaw@gmail.com> >>
 
 =head1 AUTHOR
 
